@@ -1,33 +1,31 @@
-# Import pyomo module
+# import dependencies
 from pyomo.environ import ConcreteModel, Set, Var, Objective, minimize, Constraint, log
-from pyomo.opt import SolverFactory, SolverManagerFactory
 from pyomo.core.expr.numvalue import NumericValue
 import numpy as np
 import pandas as pd
-import matplotlib.pyplot as plt
+
+from .constant import CET_ADDI, CET_MULT, FUN_PROD, FUN_COST, OPT_DEFAULT, RTS_CRS, RTS_VRS, OPT_LOCAL
+from .utils import tools
 
 
 class CNLS:
-    """Convex Nonparametric Least Square (CNLS)"""
+    """Convex Nonparametric Least Square (CNLS)
+    """
 
-    def __init__(self, y, x, z=None, cet='addi', fun='prod', rts='vrs'):
+    def __init__(self, y, x, z=None, cet=CET_ADDI, fun=FUN_PROD, rts=RTS_VRS):
+        """CNLS model
+
+        Args:
+            y (float): output variable. 
+            x (float): input variables.
+            z (float, optional): Contextual variable(s). Defaults to None.
+            cet (String, optional): CET_ADDI (additive composite error term) or CET_MULT (multiplicative composite error term). Defaults to CET_ADDI.
+            fun (String, optional): FUN_PROD (production frontier) or FUN_COST (cost frontier). Defaults to FUN_PROD.
+            rts (String, optional): RTS_VRS (variable returns to scale) or RTS_CRS (constant returns to scale). Defaults to RTS_VRS.
         """
-        Initialize the CNLS model
-
-        * y: Output variable 
-        * x: Input variables
-        * z: Contextual variables              
-        * cet = "addi" : Additive composite error term
-              = "mult" : Multiplicative composite error term
-        * fun = "prod" : Production frontier
-              = "cost" : Cost frontier
-        * rts = "vrs"  : Variable returns to scale
-              = "crs"  : Constant returns to scale
-        """
-
         # TODO(error/warning handling): Check the configuration of the model exist
-        self.x = x.tolist()
-        self.y = y.tolist()
+        self.x = tools.trans_list(x)
+        self.y = tools.trans_list(y)
         self.z = z
         self.cet = cet
         self.fun = fun
@@ -35,7 +33,7 @@ class CNLS:
 
         if type(self.x[0]) != list:
             self.x = []
-            for x_value in x.tolist():
+            for x_value in tools.trans_list(x):
                 self.x.append([x_value])
 
         if type(self.y[0]) == list:
@@ -45,10 +43,10 @@ class CNLS:
         self.__model__ = ConcreteModel()
 
         if type(self.z) != type(None):
-            self.z = z.tolist()
+            self.z = tools.trans_list(z)
             if type(self.z[0]) != list:
                 self.z = []
-                for z_value in z.tolist():
+                for z_value in tools.trans_list(z):
                     self.z.append([z_value])
 
             # Initialize the set of z
@@ -79,7 +77,7 @@ class CNLS:
         self.__model__.regression_rule = Constraint(self.__model__.I,
                                                     rule=self.__regression_rule(),
                                                     doc='regression equation')
-        if self.cet == "mult":
+        if self.cet == CET_MULT:
             self.__model__.log_rule = Constraint(self.__model__.I,
                                                  rule=self.__log_rule(),
                                                  doc='log-transformed regression equation')
@@ -98,34 +96,16 @@ class CNLS:
             rl.append(l[i][0])
         return rl
 
-    def optimize(self, remote=False):
-        """Optimize the function by requested method"""
-        # TODO(error/warning handling): Check problem status after optimization
-        if remote == False:
-            if self.cet == "addi":
-                solver = SolverFactory("mosek")
-                print("Estimating the additive model locally with mosek solver")
-                self.problem_status = solver.solve(self.__model__, tee=True)
-                self.optimization_status = 1
+    def optimize(self, email=OPT_LOCAL, solver=OPT_DEFAULT):
+        """Optimize the function by requested method
 
-            elif self.cet == "mult":
-                # TODO(warning handling): Use log system instead of print()
-                print(
-                    "Estimating the multiplicative model will be available in near future."
-                )
-                return False
-        else:
-            if self.cet == "addi":
-                opt = "mosek"
-                print("Estimating the additive model remotely with mosek solver")
-            elif self.cet == "mult":
-                opt = "knitro"
-                print("Estimating the multiplicative model remotely with knitro solver")
-            solver = SolverManagerFactory('neos')
-            self.problem_status = solver.solve(self.__model__,
-                                               tee=True,
-                                               opt=opt)
-            self.optimization_status = 1
+        Args:
+            email (string): The email address for remote optimization. It will optimize locally if OPT_LOCAL is given.
+            solver (string): The solver chosen for optimization. It will optimize with default solver if OPT_DEFAULT is given.
+        """
+        # TODO(error/warning handling): Check problem status after optimization
+        self.problem_status, self.optimization_status = tools.optimize_model(
+            self.__model__, email, self.cet, solver)
 
     def __objective_rule(self):
         """Return the proper objective function"""
@@ -137,14 +117,15 @@ class CNLS:
 
     def __regression_rule(self):
         """Return the proper regression constraint"""
-        if self.cet == "addi":
-            if self.rts == "vrs":
+        if self.cet == CET_ADDI:
+            if self.rts == RTS_VRS:
                 if type(self.z) != type(None):
                     def regression_rule(model, i):
                         return self.y[i] == model.alpha[i] \
                             + sum(model.beta[i, j] * self.x[i][j] for j in model.J) \
                             + sum(model.lamda[k] * self.z[i][k]
                                   for k in model.K) + model.epsilon[i]
+
                     return regression_rule
 
                 def regression_rule(model, i):
@@ -153,20 +134,22 @@ class CNLS:
                         + model.epsilon[i]
 
                 return regression_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
                 # TODO(warning handling): replace with model requested not exist
                 return False
 
-        elif self.cet == "mult":
+        elif self.cet == CET_MULT:
             if type(self.z) != type(None):
                 def regression_rule(model, i):
                     return log(self.y[i]) == log(model.frontier[i] + 1) \
                         + sum(model.lamda[k] * self.z[i][k]
                               for k in model.K) + model.epsilon[i]
+
                 return regression_rule
 
             def regression_rule(model, i):
                 return log(self.y[i]) == log(model.frontier[i] + 1) + model.epsilon[i]
+
             return regression_rule
 
         # TODO(error handling): replace with undefined model attribute
@@ -174,15 +157,15 @@ class CNLS:
 
     def __log_rule(self):
         """Return the proper log constraint"""
-        if self.cet == "mult":
-            if self.rts == "vrs":
+        if self.cet == CET_MULT:
+            if self.rts == RTS_VRS:
 
                 def log_rule(model, i):
                     return model.frontier[i] == model.alpha[i] + sum(
                         model.beta[i, j] * self.x[i][j] for j in model.J) - 1
 
                 return log_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
 
                 def log_rule(model, i):
                     return model.frontier[i] == sum(
@@ -195,13 +178,13 @@ class CNLS:
 
     def __afriat_rule(self):
         """Return the proper afriat inequality constraint"""
-        if self.fun == "prod":
+        if self.fun == FUN_PROD:
             __operator = NumericValue.__le__
-        elif self.fun == "cost":
+        elif self.fun == FUN_COST:
             __operator = NumericValue.__ge__
 
-        if self.cet == "addi":
-            if self.rts == "vrs":
+        if self.cet == CET_ADDI:
+            if self.rts == RTS_VRS:
 
                 def afriat_rule(model, i, h):
                     if i == h:
@@ -213,11 +196,11 @@ class CNLS:
                                              for j in model.J))
 
                 return afriat_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
                 # TODO(warning handling): replace with model requested not exist
                 return False
-        elif self.cet == "mult":
-            if self.rts == "vrs":
+        elif self.cet == CET_MULT:
+            if self.rts == RTS_VRS:
 
                 def afriat_rule(model, i, h):
                     if i == h:
@@ -229,7 +212,7 @@ class CNLS:
                                              for j in model.J))
 
                 return afriat_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
 
                 def afriat_rule(model, i, h):
                     if i == h:
@@ -330,12 +313,12 @@ class CNLS:
         if self.optimization_status == 0:
             print("Model isn't optimized. Use optimize() method to estimate the model.")
             return False
-        if self.cet == "mult" and type(self.z) == type(None):
-            frontier = np.asarray(list(self.__model__.frontier[:].value))+1
-        elif self.cet == "mult" and type(self.z) != type(None):
+        if self.cet == CET_MULT and type(self.z) == type(None):
+            frontier = np.asarray(list(self.__model__.frontier[:].value)) + 1
+        elif self.cet == CET_MULT and type(self.z) != type(None):
             frontier = list(np.divide(self.y, np.exp(
-                self.get_residual()+self.get_lamda()*np.asarray(self.z)[:, 0])) - 1)
-        elif self.cet == "addi":
+                self.get_residual() + self.get_lamda() * np.asarray(self.z)[:, 0])) - 1)
+        elif self.cet == CET_ADDI:
             frontier = np.asarray(self.y) - self.get_residual()
         return np.asarray(frontier)
 
@@ -352,46 +335,3 @@ class CNLS:
             print("Model isn't optimized. Use optimize() method to estimate the model.")
             return False
         return self.get_alpha() + np.amax(self.get_residual())
-
-    def plot2d(self, xselect, fig_name=None):
-        """
-        Plot with Selected x
-
-        * xselect: The index of selected x
-        * fig_name: The name of figure to save
-                    If `fig_name` is not given, the figure will be showed.
-        """
-        x = np.array(self.x).T[xselect]
-        y = np.array(self.y).T
-        f = y - self.get_residual()
-        data = (np.stack([x, y, f], axis=0)).T
-
-        # sort
-        data = data[np.argsort(data[:, 0])].T
-
-        x, y, f = data[0], data[1], data[2]
-
-        # create figure and axes objects
-        fig, ax = plt.subplots()
-        dp = ax.scatter(x, y, color="k", marker='x')
-        fl = ax.plot(x, f, color="r", label="CNLS")
-
-        # add legend
-        legend = plt.legend([dp, fl[0]],
-                            ['Data points', 'CNLS'],
-                            loc='upper left',
-                            ncol=1,
-                            fontsize=10,
-                            frameon=False)
-
-        # add x, y label
-        ax.set_xlabel("Input $x$%d" % (xselect))
-        ax.set_ylabel("Output $y$")
-
-        # Remove top and right axes
-        ax.spines["right"].set_visible(False)
-        ax.spines["top"].set_visible(False)
-        if fig_name == None:
-            plt.show()
-        else:
-            plt.savefig(fig_name)
