@@ -1,32 +1,33 @@
-from . import CNLS
-
+# import dependencies
 from pyomo.environ import ConcreteModel, Set, Var, Objective, minimize, Constraint
-from pyomo.opt import SolverFactory, SolverManagerFactory
 from pyomo.core.expr.numvalue import NumericValue
 import pandas as pd
 import numpy as np
 
+from . import CNLS
+from .constant import CET_ADDI, FUN_COST, FUN_PROD, OPT_DEFAULT, OPT_LOCAL
+from .utils import tools
+
 
 class CNLSDDF(CNLS.CNLS):
-    """Convex Nonparametric Least Square with multiple Outputs (DDF formulation)"""
+    """Convex Nonparametric Least Square with DDF formulation
+    """
 
-    def __init__(self, y, x, b=None, gy=[1], gx=[1], gb=None, fun='prod'):
+    def __init__(self, y, x, b=None, gy=[1], gx=[1], gb=None, fun=FUN_PROD):
+        """CNLS DDF model
+
+        Args:
+            y (float): output variable.
+            x (float): input variables.
+            b (float), optional): undesirable output variables. Defaults to None.
+            gy (list, optional): output directional vector. Defaults to [1].
+            gx (list, optional): input directional vector. Defaults to [1].
+            gb (list, optional): undesirable output directional vector. Defaults to None.
+            fun (String, optional): FUN_PROD (production frontier) or FUN_COST (cost frontier). Defaults to FUN_PROD.
         """
-        Initialize the CNLSDDF model
-
-        * y: Output variables
-        * x: Input variables
-        * b: Undesirable output variables
-        * gy: Output directional vector
-        * gx: Input directional vector
-        * gb: Undesirable output directional vector
-        * fun = "prod": Production frontier
-              = "cost": Cost frontier
-        """
-
         # TODO(error/warning handling): Check the configuration of the model exist
-        self.y = y.tolist()
-        self.x = x.tolist()
+        self.y = tools.trans_list(y)
+        self.x = tools.trans_list(x)
         self.b = b
         self.gy = self.__to_1d_list(gy)
         self.gx = self.__to_1d_list(gx)
@@ -36,12 +37,12 @@ class CNLSDDF(CNLS.CNLS):
 
         if type(self.x[0]) != list:
             self.x = []
-            for x_value in x.tolist():
+            for x_value in tools.trans_list(x):
                 self.x.append([x_value])
 
         if type(self.y[0]) != list:
             self.y = []
-            for y_value in y.tolist():
+            for y_value in ytools.trans_list():
                 self.y.append([y_value])
 
         self.__model__ = ConcreteModel()
@@ -60,12 +61,12 @@ class CNLSDDF(CNLS.CNLS):
             self.__model__.I, self.__model__.K, bounds=(0.0, None), doc='gamma')
 
         if type(self.b) != type(None):
-            self.b = b.tolist()
+            self.b = tools.trans_list(b)
             self.gb = self.__to_1d_list(gb)
 
             if type(self.b[0]) != list:
                 self.b = []
-                for b_value in b.tolist():
+                for b_value in tools.trans_list(b):
                     self.b.append([b_value])
 
             self.__model__.L = Set(initialize=range(len(self.b[0])))
@@ -91,25 +92,16 @@ class CNLSDDF(CNLS.CNLS):
         self.optimization_status = 0
         self.problem_status = 0
 
-    def optimize(self, remote=False):
-        """Optimize the function by requested method"""
-        # TODO(error/warning handling): Check problem status after optimization
-        if remote == False:
-            print("Estimating the model locally with mosek solver")
-            solver = SolverFactory("mosek")
-            self.problem_status = solver.solve(self.__model__,
-                                               tee=True)
-            print(self.problem_status)
-            self.optimization_status = 1
+    def optimize(self, email=OPT_LOCAL, solver=OPT_DEFAULT):
+        """Optimize the function by requested method
 
-        else:
-            print("Estimating the model remotely with mosek solver")
-            solver = SolverManagerFactory('neos')
-            self.problem_status = solver.solve(self.__model__,
-                                               tee=True,
-                                               opt="mosek")
-            print(self.problem_status)
-            self.optimization_status = 1
+        Args:
+            email (string): The email address for remote optimization. It will optimize locally if OPT_LOCAL is given.
+            solver (string): The solver chosen for optimization. It will optimize with default solver if OPT_DEFAULT is given.
+        """
+        # TODO(error/warning handling): Check problem status after optimization
+        self.problem_status, self.optimization_status = tools.optimize_model(
+            self.__model__, email, CET_ADDI, solver)
 
     def __to_1d_list(self, l):
         if type(l) == int or type(l) == float:
@@ -121,18 +113,18 @@ class CNLSDDF(CNLS.CNLS):
         """Return the proper regression constraint"""
         if type(self.b) == type(None):
             def regression_rule(model, i):
-                return sum(model.gamma[i, k] * self.y[i][k] for k in model.K)\
-                    == model.alpha[i]\
-                    + sum(model.beta[i, j] * self.x[i][j] for j in model.J)\
+                return sum(model.gamma[i, k] * self.y[i][k] for k in model.K) \
+                    == model.alpha[i] \
+                    + sum(model.beta[i, j] * self.x[i][j] for j in model.J) \
                     - model.epsilon[i]
 
             return regression_rule
 
         def regression_rule(model, i):
-            return sum(model.gamma[i, k] * self.y[i][k] for k in model.K)\
-                == model.alpha[i]\
-                + sum(model.beta[i, j] * self.x[i][j] for j in model.J)\
-                + sum(model.delta[i, l] * self.b[i][l] for l in model.L)\
+            return sum(model.gamma[i, k] * self.y[i][k] for k in model.K) \
+                == model.alpha[i] \
+                + sum(model.beta[i, j] * self.x[i][j] for j in model.J) \
+                + sum(model.delta[i, l] * self.b[i][l] for l in model.L) \
                 - model.epsilon[i]
 
         return regression_rule
@@ -143,6 +135,7 @@ class CNLSDDF(CNLS.CNLS):
             def translation_rule(model, i):
                 return sum(model.beta[i, j] * self.gx[j] for j in model.J) \
                     + sum(model.gamma[i, k] * self.gy[k] for k in model.K) == 1
+
             return translation_rule
 
         def translation_rule(model, i):
@@ -154,9 +147,9 @@ class CNLSDDF(CNLS.CNLS):
 
     def __afriat_rule(self):
         """Return the proper afriat inequality constraint"""
-        if self.fun == "prod":
+        if self.fun == FUN_PROD:
             __operator = NumericValue.__le__
-        elif self.fun == "cost":
+        elif self.fun == FUN_COST:
             __operator = NumericValue.__ge__
 
         if type(self.b) == type(None):
