@@ -1,32 +1,32 @@
-# Import pyomo module
+# import dependencies
 from pyomo.environ import ConcreteModel, Set, Var, Objective, minimize, Constraint, log
-from pyomo.opt import SolverFactory, SolverManagerFactory
 from pyomo.core.expr.numvalue import NumericValue
 import numpy as np
 import pandas as pd
-import matplotlib.pyplot as plt
+
+from .constant import CET_ADDI, CET_MULT, FUN_PROD, FUN_COST, RTS_CRS, RTS_VRS, OPT_LOCAL, OPT_DEFAULT
+from .utils import tools
 
 
 class CQR:
-    """Convex quantile regression (CQR)"""
+    """Convex quantile regression (CQR)
+    """
 
-    def __init__(self, y, x, tau, z=None, cet='addi', fun='prod', rts='vrs'):
-        """
-          * y : Output variable
-          * x : Input variables
-          * z: Contextual variables  
-          * tau : quantile
-          * cet  = "addi" : Additive composite error term
-                 = "mult" : Multiplicative composite error term
-          * fun  = "prod" : Production frontier
-                 = "cost" : Cost frontier
-          * rts  = "vrs"  : Variable returns to scale
-                 = "crs"  : Constant returns to scale
-        """
+    def __init__(self, y, x, tau, z=None, cet=CET_ADDI, fun=FUN_PROD, rts=RTS_VRS):
+        """CQR model
 
+        Args:
+            y (float): output variable. 
+            x (float): input variables.
+            z (float, optional): Contextual variable(s). Defaults to None.
+            tau (float): quantile.
+            cet (String, optional): CET_ADDI (additive composite error term) or CET_MULT (multiplicative composite error term). Defaults to CET_ADDI.
+            fun (String, optional): FUN_PROD (production frontier) or FUN_COST (cost frontier). Defaults to FUN_PROD.
+            rts (String, optional): RTS_VRS (variable returns to scale) or RTS_CRS (constant returns to scale). Defaults to RTS_VRS.
+        """
         # TODO(error/warning handling): Check the configuration of the model exist
-        self.x = x.tolist()
-        self.y = y.tolist()
+        self.x = tools.trans_list(x)
+        self.y = tools.trans_list(y)
         self.z = z
         self.tau = tau
         self.cet = cet
@@ -35,7 +35,7 @@ class CQR:
 
         if type(self.x[0]) != list:
             self.x = []
-            for x_value in x.tolist():
+            for x_value in tools.trans_list(x):
                 self.x.append([x_value])
 
         if type(self.y[0]) == list:
@@ -45,10 +45,10 @@ class CQR:
         self.__model__ = ConcreteModel()
 
         if type(self.z) != type(None):
-            self.z = z.tolist()
+            self.z = tools.trans_list(z)
             if type(self.z[0]) != list:
                 self.z = []
-                for z_value in z.tolist():
+                for z_value in tools.trans_list(z):
                     self.z.append([z_value])
 
             # Initialize the set of z
@@ -64,22 +64,22 @@ class CQR:
         # Initialize the variables
         self.__model__.alpha = Var(self.__model__.I, doc='alpha')
         self.__model__.beta = Var(self.__model__.I,
-                                self.__model__.J,
-                                bounds=(0.0, None),
-                                doc='beta')
+                                  self.__model__.J,
+                                  bounds=(0.0, None),
+                                  doc='beta')
         self.__model__.epsilon = Var(self.__model__.I, doc='error term')
         self.__model__.epsilon_plus = Var(
             self.__model__.I, bounds=(0.0, None), doc='positive error term')
         self.__model__.epsilon_minus = Var(
             self.__model__.I, bounds=(0.0, None), doc='negative error term')
         self.__model__.frontier = Var(self.__model__.I,
-                                    bounds=(0.0, None),
-                                    doc='estimated frontier')
+                                      bounds=(0.0, None),
+                                      doc='estimated frontier')
 
         # Setup the objective function and constraints
         self.__model__.objective = Objective(rule=self.__objective_rule(),
-                                            sense=minimize,
-                                            doc='objective function')
+                                             sense=minimize,
+                                             doc='objective function')
 
         self.__model__.error_decomposition = Constraint(self.__model__.I,
                                                         rule=self.__error_decomposition(),
@@ -88,10 +88,10 @@ class CQR:
         self.__model__.regression_rule = Constraint(self.__model__.I,
                                                     rule=self.__regression_rule(),
                                                     doc='regression equation')
-        if self.cet == "mult":
+        if self.cet == CET_MULT:
             self.__model__.log_rule = Constraint(self.__model__.I,
-                                                    rule=self.__log_rule(),
-                                                    doc='log-transformed regression equation')
+                                                 rule=self.__log_rule(),
+                                                 doc='log-transformed regression equation')
 
         self.__model__.afriat_rule = Constraint(self.__model__.I,
                                                 self.__model__.I,
@@ -102,34 +102,16 @@ class CQR:
         self.optimization_status = 0
         self.problem_status = 0
 
-    def optimize(self, remote=True):
-        """Optimize the function by requested method"""
-        # TODO(error/warning handling): Check problem status after optimization
-        if remote == False:
-            if self.cet == "addi":
-                solver = SolverFactory("mosek")
-                print("Estimating the additive model locally with mosek solver")
-                self.problem_status = solver.solve(self.__model__, tee=True)
-                self.optimization_status = 1
+    def optimize(self, email=OPT_LOCAL, solver=OPT_DEFAULT):
+        """Optimize the function by requested method
 
-            elif self.cet == "mult":
-                # TODO(warning handling): Use log system instead of print()
-                print(
-                    "Estimating the multiplicative model will be available in near future."
-                )
-                return False
-        else:
-            if self.cet == "addi":
-                opt = "mosek"
-                print("Estimating the additive model remotely with mosek solver")
-            elif self.cet == "mult":
-                opt = "knitro"
-                print("Estimating the multiplicative model remotely with knitro solver")
-            solver = SolverManagerFactory('neos')
-            self.problem_status = solver.solve(self.__model__,
-                                                tee=True,
-                                                opt=opt)
-            self.optimization_status = 1
+        Args:
+            email (string): The email address for remote optimization. It will optimize locally if OPT_LOCAL is given.
+            solver (string): The solver chosen for optimization. It will optimize with default solver if OPT_DEFAULT is given.
+        """
+        # TODO(error/warning handling): Check problem status after optimization
+        self.problem_status, self.optimization_status = tools.optimize_model(
+            self.__model__, email, self.cet, solver)
 
     def __to_1d_list(self, l):
         rl = []
@@ -142,7 +124,7 @@ class CQR:
 
         def objective_rule(model):
             return self.tau * sum(model.epsilon_plus[i] for i in model.I) \
-                    + (1 - self.tau) * sum(model.epsilon_minus[i] for i in model.I)
+                + (1 - self.tau) * sum(model.epsilon_minus[i] for i in model.I)
 
         return objective_rule
 
@@ -156,14 +138,15 @@ class CQR:
 
     def __regression_rule(self):
         """Return the proper regression constraint"""
-        if self.cet == "addi":
-            if self.rts == "vrs":
+        if self.cet == CET_ADDI:
+            if self.rts == RTS_VRS:
                 if type(self.z) != type(None):
                     def regression_rule(model, i):
                         return self.y[i] == model.alpha[i] \
                             + sum(model.beta[i, j] * self.x[i][j] for j in model.J) \
                             + sum(model.lamda[k] * self.z[i][k]
-                                    for k in model.K) + model.epsilon[i]
+                                  for k in model.K) + model.epsilon[i]
+
                     return regression_rule
 
                 def regression_rule(model, i):
@@ -172,20 +155,22 @@ class CQR:
                         + model.epsilon[i]
 
                 return regression_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
                 # TODO(warning handling): replace with model requested not exist
                 return False
 
-        elif self.cet == "mult":
+        elif self.cet == CET_MULT:
             if type(self.z) != type(None):
                 def regression_rule(model, i):
                     return log(self.y[i]) == log(model.frontier[i] + 1) \
-                                + sum(model.lamda[k] * self.z[i][k]
-                                        for k in model.K) + model.epsilon[i]
+                        + sum(model.lamda[k] * self.z[i][k]
+                              for k in model.K) + model.epsilon[i]
+
                 return regression_rule
 
             def regression_rule(model, i):
                 return log(self.y[i]) == log(model.frontier[i] + 1) + model.epsilon[i]
+
             return regression_rule
 
         # TODO(error handling): replace with undefined model attribute
@@ -193,15 +178,15 @@ class CQR:
 
     def __log_rule(self):
         """Return the proper log constraint"""
-        if self.cet == "mult":
-            if self.rts == "vrs":
+        if self.cet == CET_MULT:
+            if self.rts == RTS_VRS:
 
                 def log_rule(model, i):
                     return model.frontier[i] == model.alpha[i] + sum(
                         model.beta[i, j] * self.x[i][j] for j in model.J) - 1
 
                 return log_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
 
                 def log_rule(model, i):
                     return model.frontier[i] == sum(
@@ -214,40 +199,41 @@ class CQR:
 
     def __afriat_rule(self):
         """Return the proper afriat inequality constraint"""
-        if self.fun == "prod":
+        if self.fun == FUN_PROD:
             __operator = NumericValue.__le__
-        elif self.fun == "cost":
+        elif self.fun == FUN_COST:
             __operator = NumericValue.__ge__
 
-        if self.cet == "addi":
-            if self.rts == "vrs":
+        if self.cet == CET_ADDI:
+            if self.rts == RTS_VRS:
 
                 def afriat_rule(model, i, h):
                     if i == h:
                         return Constraint.Skip
                     return __operator(
                         model.alpha[i] + sum(model.beta[i, j] * self.x[i][j]
-                                                for j in model.J),
+                                             for j in model.J),
                         model.alpha[h] + sum(model.beta[h, j] * self.x[i][j]
-                                                for j in model.J))
+                                             for j in model.J))
+
                 return afriat_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
                 # TODO(warning handling): replace with model requested not exist
                 return False
-        elif self.cet == "mult":
-            if self.rts == "vrs":
+        elif self.cet == CET_MULT:
+            if self.rts == RTS_VRS:
 
                 def afriat_rule(model, i, h):
                     if i == h:
                         return Constraint.Skip
                     return __operator(
                         model.alpha[i] + sum(model.beta[i, j] * self.x[i][j]
-                                                for j in model.J),
+                                             for j in model.J),
                         model.alpha[h] + sum(model.beta[h, j] * self.x[i][j]
-                                                for j in model.J))
+                                             for j in model.J))
 
                 return afriat_rule
-            elif self.rts == "crs":
+            elif self.rts == RTS_CRS:
 
                 def afriat_rule(model, i, h):
                     if i == h:
@@ -282,6 +268,17 @@ class CQR:
             return False
         self.__model__.beta.display()
 
+    def display_lamda(self):
+        """Display lamda value"""
+        if self.optimization_status == 0:
+            print("Model isn't optimized. Use optimize() method to estimate the model.")
+            return False
+        if type(self.z) == type(None):
+            # TODO: Replace print by warning
+            print("Without z variable")
+            return
+        self.__model__.lamda.display()
+
     def display_residual(self):
         """Dispaly residual value"""
         if self.optimization_status == 0:
@@ -321,11 +318,23 @@ class CQR:
             print("Model isn't optimized. Use optimize() method to estimate the model.")
             return False
         beta = np.asarray([i + tuple([j]) for i, j in zip(list(self.__model__.beta),
-                                                            list(self.__model__.beta[:, :].value))])
+                                                          list(self.__model__.beta[:, :].value))])
         beta = pd.DataFrame(beta, columns=['Name', 'Key', 'Value'])
         beta = beta.pivot(index='Name', columns='Key', values='Value')
         return beta.to_numpy()
 
+    def get_lamda(self):
+        """Return beta value by array"""
+        if self.optimization_status == 0:
+            print("Model isn't optimized. Use optimize() method to estimate the model.")
+            return False
+        if type(self.z) == type(None):
+            # TODO: Replace print by warning
+            print("Without z variable")
+            return
+        lamda = list(self.__model__.lamda[:].value)
+        return np.asarray(lamda)
+
     def get_residual(self):
         """Return residual value by array"""
         if self.optimization_status == 0:
@@ -355,65 +364,28 @@ class CQR:
         if self.optimization_status == 0:
             print("Model isn't optimized. Use optimize() method to estimate the model.")
             return False
-        if self.cet == "mult":
-            frontier = np.asarray(list(self.__model__.frontier[:].value))+1
-        elif self.cet == "addi":
+        if self.cet == CET_MULT:
+            frontier = np.asarray(list(self.__model__.frontier[:].value)) + 1
+        elif self.cet == CET_ADDI:
             frontier = np.asarray(self.y) - self.get_residual()
         return np.asarray(frontier)
 
-    def plot2d(self, xselect, fig_name=None):
-        """Plot with selected x"""
-        x = np.array(self.x).T[xselect]
-        y = np.array(self.y).T
-        f = y - self.get_residual()
-        data = (np.stack([x, y, f], axis=0)).T
-
-        # sort
-        data = data[np.argsort(data[:, 0])].T
-
-        x, y, f = data[0], data[1], data[2]
-
-        # create figure and axes objects
-        fig, ax = plt.subplots()
-        dp = ax.scatter(x, y, color="k", marker='x')
-        fl = ax.plot(x, f, color="r", label="CNLS")
-
-        # add legend
-        legend = plt.legend([dp, fl[0]],
-                            ['Data points', 'CNLS'],
-                            loc='upper left',
-                            ncol=1,
-                            fontsize=10,
-                            frameon=False)
-
-        # add x, y label
-        ax.set_xlabel("Input $x$%d" % (xselect))
-        ax.set_ylabel("Output $y$")
-
-        # Remove top and right axes
-        ax.spines["right"].set_visible(False)
-        ax.spines["top"].set_visible(False)
-        if fig_name == None:
-            plt.show()
-        else:
-            plt.savefig(fig_name)
-
 
 class CER(CQR):
-    """Convex expectile regression (CER)"""
+    """Convex expectile regression (CER)
+    """
 
-    def __init__(self, y, x, tau, z=None, cet='addi', fun='prod', rts='vrs'):
-        """
-           * y : Output variable
-           * x : Input variables
-           * z: Contextual variables  
-           * tau : expectile
-           * cet  = "addi" : Additive composite error term
-                  = "mult" : Multiplicative composite error term
-           * fun  = "prod" : Production frontier
-                  = "cost" : Cost frontier
-           * rts  = "vrs"  : Variable returns to scale
-                  = "crs"  : Constant returns to scale
+    def __init__(self, y, x, tau, z=None, cet=CET_ADDI, fun=FUN_PROD, rts=RTS_VRS):
+        """CER model
+
+        Args:
+            y (float): output variable. 
+            x (float): input variables.
+            z (float, optional): Contextual variable(s). Defaults to None.
+            tau (float): expectile.
+            cet (String, optional): CET_ADDI (additive composite error term) or CET_MULT (multiplicative composite error term). Defaults to CET_ADDI.
+            fun (String, optional): FUN_PROD (production frontier) or FUN_COST (cost frontier). Defaults to FUN_PROD.
+            rts (String, optional): RTS_VRS (variable returns to scale) or RTS_CRS (constant returns to scale). Defaults to RTS_VRS.
         """
         super().__init__(y, x, tau, z, cet, fun, rts)
         self.__model__.objective.deactivate()
@@ -422,7 +394,8 @@ class CER(CQR):
 
     def __squared_objective_rule(self):
         def squared_objective_rule(model):
-            return self.tau * sum(model.epsilon_plus[i] ** 2 for i in model.I)\
+            return self.tau * sum(model.epsilon_plus[i] ** 2 for i in model.I) \
                 + (1 - self.tau) * \
                 sum(model.epsilon_minus[i] ** 2 for i in model.I)
+
         return squared_objective_rule
