@@ -1,12 +1,14 @@
 from . import CNLS
 from . import CNLSDDF
 from . import CNLSG
+from . import constant
 from . import CQER
 from . import CQERDDF
 from . import CQERG
 from . import dataset
 from . import DEA
 from . import FDH
+from . import plot
 from . import ICNLS
 from . import ICQER
 from . import StoNED
@@ -15,12 +17,14 @@ __all__ = [
     'CNLS',
     'CNLSDDF',
     'CNLSG',
+    'constant',
     'CQER',
     'CQERDDF',
     'CQERG',
     'dataset',
     'DEA',
     'FDH',
+    'plot',
     'ICNLS',
     'ICQER',
     'StoNED',
