@@ -6,4 +6,6 @@ from . import CQERG1
 from . import CQERG2
 from . import CQERZG1
 from . import CQERZG2
+from . import interpolation
 from . import sweet
+from . import tools
